-------------------------------------------------------------------------------
-- ElvUI Improved Spec Switch Datatext By Lockslap
-- <Borderline Amazing> US, Bleeding Hollow
-------------------------------------------------------------------------------
local addon = select(1, ...)
local E, _, V, P, G, _ = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_ISS", false)
local EP = LibStub("LibElvUIPlugin-1.0")

local lastPanel, active, activeSet
local displayString = ""
local activeString = ("|cff00ff00%s|r"):format(ACTIVE_PETS)
local inactiveString = ("|cffff0000%s|r"):format(FACTION_INACTIVE)

local function GetCurrentEquipmentSet()
	if GetNumEquipmentSets() == 0 then return false end
	for i = 1, GetNumEquipmentSets() do
		local name, _, _, isEquipped, _, _, _, _, _ = GetEquipmentSetInfo(i)
		if isEquipped then
			return name
		end
	end
	return false
end

local function OnEvent(self, event)
	lastPanel = self
	if not GetSpecializationInfo(1) then return end
	active = GetActiveSpecGroup()
	activeSet = GetCurrentEquipmentSet()
	if GetSpecialization(false, false, active) then
		self.text:SetText(select(2, GetSpecializationInfo(GetSpecialization(false, false, active))))
	end
end

local function OnEnter(self)
	DT:SetupTooltip(self)
	for i = 1, GetNumSpecGroups() do
		if GetSpecialization(false, false, i) then
			local _, name, _, icon, _, _ = GetSpecializationInfo(GetSpecialization(false, false, i))
			DT.tooltip:AddDoubleLine(displayString:format(icon, name), i == active and activeString or inactiveString, 1, 1, 1)
		end
	end
	DT.tooltip:AddLine(" ")
	DT.tooltip:AddDoubleLine(L["Left Click"], L["Change Specialization"], 1, 1, 1, 1, 1, 0)
	DT.tooltip:AddDoubleLine(L["Right Click"], L["Open Talents Frame"], 1, 1, 1, 1, 1, 0)
	DT.tooltip:Show()
end

local function OnClick(self, button)
	if button == "LeftButton" then
		SetActiveSpecGroup(active == 1 and 2 or 1)
	else
		ToggleTalentFrame()
	end
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = string.join("", "|T%s:12:12:1:0|t |cffffffff%s:|r")
	if lastPanel ~= nil then OnEvent(lastPanel) end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

P["impss"] = {
	["switch"] = false,
	["primary"] = "none",
	["secondary"] = "none",
}

local function InjectOptions()
	E.Options.args.datatexts.args.specswitch = {
		type = "group",
		order = 24,
		name = L["Improved Spec Switch"],
		disabled = function() return GetNumEquipmentSets() == 0 end,
		guiInline = true,
		get = function(info) return E.db.impss[info[#info]] end,
		set = function(info, value) E.db.impss[info[#info]] = value end,
		args = {
			switch = {
				type = "toggle",
				order = 1,
				name = L["Swap Equipment Sets"],
				desc = L["Change equipment sets when you change your spec."],
				disabled = function()
					if not GetSpecialization(false, false, 1) or not GetSpecialization(false, false, 2) then 
						return true
					else
						return false
					end
				end,
			},
			primary = {
				type = "select",
				order = 2,
				name = GetSpecialization(false, false, 1) and select(2, GetSpecializationInfo(GetSpecialization(false, false, 1))) or L["Primary Talents"],
				desc = L["Choose the equipment set to use for your primary spec."],
				disabled = function() return not E.db.impss.switch end,
				values = function()
					local sets = {
						["none"] = L["No Change"],
					}
					for i = 1, GetNumEquipmentSets() do
						local name, _, _, _, _, _, _, _, _ = GetEquipmentSetInfo(i)
						if name then
							sets[name] = name
						end
					end
					sort(sets, function(a, b) return a < b end)
					return sets
				end,
			},
			secondary = {
				type = "select",
				order = 3,
				name = GetSpecialization(false, false, 2) and select(2, GetSpecializationInfo(GetSpecialization(false, false, 2))) or L["Secondary Talents"],
				desc = L["Choose the equipment set to use for your secondary spec."],
				disabled = function() return not E.db.impss.switch end,
				values = function()
					local sets = {
						["none"] = L["No Change"],
					}
					for i = 1, GetNumEquipmentSets() do
						local name, _, _, _, _, _, _, _, _ = GetEquipmentSetInfo(i)
						if name then
							sets[name] = name
						end
					end
					sort(sets, function(a, b) return a < b end)
					return sets
				end,
			},
		},
	}
end

local f = CreateFrame("Frame")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
f:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		EP:RegisterPlugin(addon, InjectOptions)
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		
	--[[for some reason elvui's datatext module isn't firing events properly
		or isnt reporting what even just fired, which causes the equipment sets
		not to be equipped when they should and the average person may see that as a bug.
		so until i can figure out something better we're gonna do it this way.]]
	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		-- determine if we need to change the equipment set
		if E.db.impss.switch and GetNumEquipmentSets() > 0 then
			local set = active == 1 and E.db.impss.primary or E.db.impss.secondary	-- the set that should be equipped
			if set ~= "none" and set ~= activeSet then
				UseEquipmentSet(set)
			end
		end
	end
end)

DT:RegisterDatatext(L["Improved Spec Switch"], {"PLAYER_ENTERING_WORLD", "CHARACTER_POINTS_CHANGED", "PLAYER_TALENT_UPDATE", "ACTIVE_TALENT_GROUP_CHANGED"}, OnEvent, nil, OnClick, OnEnter)